using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace Monitor
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// Store the MSX-E object
        /// </summary>
        private Msxexxxx msxe;

        /// <summary>
        /// Data client
        /// </summary>
        private DataClient client;

        /// <summary>
        /// Writer to create the CSV file
        /// </summary>
        private TextWriter writer;

        /// <summary>
        /// Constructor
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Event handler: occurs once when the form is loaded
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            // Load default parameters
            this.ipTextBox.Text = Properties.Settings.Default.ip;
        }

        /// <summary>
        /// Event handler: occurs once when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Event handler: click on the "Initialise connection" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void initialiseConnectionButton_Click(object sender, EventArgs e)
        {
            // Check the validity of the IP address
            if (this.ipTextBox.Valid == false)
                return;

            // Update some graphics
            this.initialiseConnectionButton.Enabled = false;
            this.actionToolStripProgressBar.Visible = true;
            this.actionToolStripStatusLabel.Text = "Initialise the connection with the MSX-E system.";
            this.actionToolStripStatusLabel.Visible = true;

            // Start the connection background worker
            this.connectionBackgroundWorker.RunWorkerAsync(this.ipTextBox.Text);
        }

        /// <summary>
        /// Job of the connection background worker
        /// The connection task is done in background since it may takes some time to initialise the system
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectionBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            int retval = 0;
            string error = string.Empty;

            // Construct the new MSX-E object
            Msxexxxx m = new Msxexxxx(this.ipTextBox.Text);

            // Do the connection
            retval = m.Connect(out error);
            e.Result = new ConnectionWorkerResult(m, retval, error);
        }

        /// <summary>
        /// Event handler: Connection background worker is completed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectionBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ConnectionWorkerResult r = (ConnectionWorkerResult)e.Result;

            // Error ?
            if (r.retval != 0)
            {
                string error = string.Empty;
                this.actionToolStripProgressBar.Visible = false;
                this.actionToolStripStatusLabel.Text = "Error occurred.";
                // Reset the system type
                this.systemInformationTextBox.Text = string.Empty;
                // Display the right text
                switch (r.retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + r.error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + r.error;
                        break;
                }
                // Display the error
                MessageBox.Show(error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                // Store the object
                this.msxe = r.msxe;

                this.actionToolStripProgressBar.Visible = false;
                this.actionToolStripStatusLabel.Text = "Connection opened with the system: " + r.msxe.IPAddress;
                // Display the system type
                this.systemInformationTextBox.Text = r.msxe.SystemType;

                // Graphics
                this.acquisitionGroupBox.Enabled = true;
                this.channelsListView.Items.Clear();
                this.valueListView.Items.Clear();

                // Fill the channels
                for (int i = 0; i < (int)this.msxe.AcquisitionChannelNb; i++)
                {
                    ListViewItem item = new ListViewItem(new string[] { string.Empty, i.ToString(), this.msxe.GetAcquisitionChannelType(i) });
                    item.Checked = true;
                    channelsListView.Items.Add(item);

                    ListViewItem valueItem = new ListViewItem(new string[] { i.ToString(), "Waiting data..." });
                    valueListView.Items.Add(valueItem);
                }
            }

            // Enable the "Initialise Connection" button
            this.initialiseConnectionButton.Enabled = true;
        }

        /// <summary>
        /// Event handler: click on the "Browse" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void browseButton_Click(object sender, EventArgs e)
        {
            SaveFileDialog d = new SaveFileDialog();
            d.Filter = "CSV Files |*.csv";
            if (d.ShowDialog() != DialogResult.OK)
            {
                this.fileTextBox.Text = string.Empty;
                return;
            }

            // Store it
            this.fileTextBox.Text = d.FileName;
        }

        /// <summary>
        /// Return the selected channels
        /// </summary>
        /// <returns></returns>
        private List<int> GetSelectedChannels()
        {
            List<int> ch = new List<int>();
            for (int i = 0; i < (int)this.msxe.AcquisitionChannelNb; i++)
            {
                if (channelsListView.Items[i].Checked)
                    ch.Add(i);
            }
            return ch;
        }

        /// <summary>
        /// Event handler: Click on the start button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startButton_Click(object sender, EventArgs e)
        {
            List<int> channels = null;
            int retval = 0;
            string error = string.Empty;

            // Check if file is selected
            if (this.fileTextBox.Text == String.Empty)
            {
                MessageBox.Show("Please select a file in the recording part before starting.", 
                    "Error", 
                    MessageBoxButtons.OK, 
                    MessageBoxIcon.Error);
                return;
            }

            // Get selected channels
            channels = this.GetSelectedChannels();

            // Create the file
            try
            {
                this.writer = new StreamWriter(this.fileTextBox.Text);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Cannot open the file: " + this.fileTextBox.Text + ", exception: " + ex.Message,
                    "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                return;
            }

            // Generate the header
            try
            {
                this.writer.Write("Timestamp seconds; Timestamp micro-seconds");
                for (int i = 0; i < channels.Count; i++)
                    this.writer.Write("; Channel" + channels[i].ToString());

                this.writer.Write(Environment.NewLine);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Cannot write to the file: " + this.fileTextBox.Text + ", exception: " + ex.Message,
                    "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                return;
            }
            
            // Open data client
            this.client = new DataClient(this.msxe.IPAddress, 8989, 8 + 4 * channels.Count);
            if (this.client.StartListening() != 0)
            {
                MessageBox.Show("An error occured while opening socket connection on port 8989.",
                    "Error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                writer.Close();
                return;
            }

            // Wait to be sure that the connection is opened
            System.Threading.Thread.Sleep(100);

            // Start the sequence
            retval = this.msxe.AcquisitionSequenceInitAndStart(channels, out error);
            if (retval != 0)
            {
                switch (retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + error;
                        break;
                }
                // Display the error
                MessageBox.Show(error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                writer.Close();
                client.Close();
                return;
            }

            // Graphics
            this.systemConnectionGroupBox.Enabled = false;
            this.stopButton.Enabled = true;
            this.startButton.Enabled = false;
            this.browseButton.Enabled = false;

            this.actionToolStripProgressBar.Visible = true;
            this.actionToolStripStatusLabel.Text = "Receiving data...";
            this.actionToolStripStatusLabel.Visible = true;

            // Start worker
            this.writerBackgroundWorker.RunWorkerAsync(new WriterParams(writer, client, channels));
        }

        /// <summary>
        /// Event handler: Click on the stop button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopButton_Click(object sender, EventArgs e)
        {
            // Close
            this.msxe.StopAll();
            this.stopButton.Enabled = false;
            this.writerBackgroundWorker.CancelAsync();

            // Fill with waiting data....
            for (int i = 0; i < (int)msxe.AcquisitionChannelNb; i++)
            {
                valueListView.Items[i].SubItems[1].Text = "Waiting data...";
            }
        }

        /// <summary>
        /// Job of the writer
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void writerBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = (BackgroundWorker)sender;
            WriterParams p = (WriterParams)e.Argument;

            // Endless loop
            for (; ; )
            {
                bool doStop = false;
                bool first = true;

                // May stop?
                if (worker.CancellationPending)
                    doStop = true;

                // While data are available
                while (true)
                {
                    DataClientEventArgs a = p.client.PopPacket();
                    if (a == null)
                        break;

                    // Time stamp
                    UInt32 ts = BitConverter.ToUInt32(a.data, 0);
                    UInt32 tus = BitConverter.ToUInt32(a.data, 4);

                    try
                    {
                        p.writer.Write(ts.ToString() + ";" + tus.ToString());
                        // Encoded on float
                        for (int i = 0; i < p.channels.Count; i++)
                        {
                            float value = BitConverter.ToSingle(a.data, 8 + i * 4);
                            p.writer.Write(";" + value.ToString("F4"));
                        }
                        p.writer.Write(Environment.NewLine);
                    }
                    catch ( Exception ex )
                    {
                        MessageBox.Show("Cannot write to the file, exception: " + ex.Message,
                            "Error",
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error);
                        return;
                    }

                    // Report the values
                    if (first)
                    {
                        worker.ReportProgress(1, a);
                    }

                    first = false;
                }

                if (doStop)
                    return;

                System.Threading.Thread.Sleep(50);
            }
        }

        /// <summary>
        /// Writer is completed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void writerBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.client.Close();
            this.writer.Close();
            this.startButton.Enabled = true;
            this.browseButton.Enabled = true;
            this.systemConnectionGroupBox.Enabled = true;

            this.actionToolStripProgressBar.Visible = false;
            this.actionToolStripStatusLabel.Text = "Connection opened with the system: " + msxe.IPAddress;
        }

        /// <summary>
        /// Worker report
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void writerBackgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            DataClientEventArgs a = (DataClientEventArgs)e.UserState;
            for (int i = 0; i < ((a.data.Length - 8) / 4); i++)
            {
                float value = BitConverter.ToSingle(a.data, 8 + i * 4);
                this.valueListView.Items[i].SubItems[1].Text = value.ToString("F4");
            }
        }
    }
}