using System;
using System.Collections.Generic;
using System.Text;

namespace ReadDigitalInputs
{
    /// <summary>
    /// Represent a digital I/O
    /// </summary>
    public class DigitalIO
    {
        /// <summary>
        /// Related digital I/O port
        /// </summary>
        private ulong port;

        /// <summary>
        /// Index on that port
        /// </summary>
        private ulong relativeIndex;

        /// <summary>
        /// Global index
        /// </summary>
        private ulong index;

        /// <summary>
        /// True if the I/O can be configured as input
        /// </summary>
        private bool inputPossible;

        /// <summary>
        /// True if the I/O can be configured as output
        /// </summary>
        private bool outputPossible;

        /// <summary>
        /// Current configuration
        /// 0: input
        /// 1: output
        /// </summary>
        private ulong currentConfiguration;

        /// <summary>
        /// Current state
        /// 0: low
        /// 1: high
        /// </summary>
        private ulong currentState;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="index"></param>
        /// <param name="port"></param>
        /// <param name="relativeIndex"></param>
        /// <param name="inputPossible"></param>
        /// <param name="outputPossible"></param>
        /// <param name="currentConfiguration"></param>
        /// <param name="currentState"></param>
        public DigitalIO(ulong index,
            ulong port,
            ulong relativeIndex,
            bool inputPossible,
            bool outputPossible,
            ulong currentConfiguration,
            ulong currentState)
        {
            this.index = index;
            this.port = port;
            this.relativeIndex = relativeIndex;
            this.inputPossible = inputPossible;
            this.outputPossible = outputPossible;
            this.currentConfiguration = currentConfiguration;
            this.currentState = currentState;
        }

        /// <summary>
        /// Return the global index of the digital I/O
        /// </summary>
        public ulong Index
        {
            get { return this.index; }
        }

        /// <summary>
        /// Return the related port
        /// </summary>
        public ulong Port
        {
            get { return this.port; }
        }

        /// <summary>
        /// Return the index of the I/O inside the port
        /// </summary>
        public ulong RelativeIndex
        {
            get { return this.relativeIndex; }
        }

        /// <summary>
        /// Return true if the digital I/O can be configured as an input
        /// </summary>
        public bool InputPossible
        {
            get { return this.inputPossible; }
        }

        /// <summary>
        /// Return true if the digital I/O can be configured as an output
        /// </summary>
        public bool OutputPossible
        {
            get { return this.outputPossible; }
        }

        /// <summary>
        /// Return the current configuration
        /// 0: input
        /// 1: output
        /// </summary>
        public ulong CurrentConfiguration
        {
            get { return this.currentConfiguration; }
        }

        /// <summary>
        /// Return the current state
        /// 0: low
        /// 1: high
        /// </summary>
        public ulong CurrentState
        {
            get { return this.currentState; }
            set
            {
                if (value != 0)
                    this.currentState = 1;
                else
                    this.currentState = 0;
            }
        }
    }
}
