using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace ReadDigitalInputs
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// Store the MSX-E object
        /// </summary>
        private Msxexxxx msxe;

        /// <summary>
        /// Constructor of the graphical interface
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Event handler: occurs once when the form is loaded
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            // Load default parameters
            this.ipTextBox.Text = Properties.Settings.Default.ip;
            this.refreshTimeTextBox.Text = Properties.Settings.Default.refreshTime;
        }

        /// <summary>
        /// Event handler: occurs when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.refreshTime = this.refreshTimeTextBox.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Event handler: click on the "Initialise connection" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void initialiseConnectionButton_Click(object sender, EventArgs e)
        {
            // Check the validity of the IP address
            if (this.ipTextBox.Valid == false)
                return;

            // Update some graphics
            this.initialiseConnectionButton.Enabled = false;
            this.actionToolStripProgressBar.Visible = true;
            this.actionToolStripStatusLabel.Text = "Initialise the connection with the MSX-E system.";
            this.actionToolStripStatusLabel.Visible = true;

            this.digitalIOGroupBox.Enabled = false;
            this.digitalIODataGridView.Rows.Clear();

            // Start the connection background worker
            this.connectionBackgroundWorker.RunWorkerAsync(this.ipTextBox.Text);
        }

        /// <summary>
        /// Job of the connection background worker
        /// The connection task is done in background since it may takes some time to initialise the system
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectionBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            int retval = 0;
            string error = string.Empty;

            // Construct the new MSX-E object
            Msxexxxx m = new Msxexxxx(this.ipTextBox.Text);

            // Do the connection
            retval = m.Connect(out error);
            e.Result = new ConnectionWorkerResult(m, retval, error);
        }

        /// <summary>
        /// Event handler: Connection background worker is completed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectionBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ConnectionWorkerResult r = (ConnectionWorkerResult)e.Result;

            // Error ?
            if (r.retval != 0)
            {
                string error = string.Empty;
                this.actionToolStripProgressBar.Visible = false;
                this.actionToolStripStatusLabel.Text = "Error occurred.";
                // Reset the system type
                this.systemInformationTextBox.Text = string.Empty;
                // Display the right text
                switch (r.retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + r.error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + r.error;
                        break;
                }
                // Display the error
                MessageBox.Show(error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                // Store the object
                this.msxe = r.msxe;

                this.actionToolStripProgressBar.Visible = false;
                this.actionToolStripStatusLabel.Text = "Connection opened with the system: " + r.msxe.IPAddress;
                // Display the system type
                this.systemInformationTextBox.Text = r.msxe.SystemType;

                // Enable controls
                this.digitalIOGroupBox.Enabled = true;
                
                // Set data grid
                for (int i = 0; i < (int)this.msxe.DigIOChannelNb; i++)
                {
                    DigitalIO info = this.msxe.GetDigitalIoInfo(i);

                    // Only input
                    if (info.CurrentConfiguration != 0)
                        continue;

                    int index = digitalIODataGridView.Rows.Add();
                    digitalIODataGridView.Rows[index].Cells[0].Value = i.ToString();
                    digitalIODataGridView.Rows[index].Cells[1].Value = info.CurrentState.ToString();
                    digitalIODataGridView.Rows[index].Cells[2].Value = info.Port.ToString();

                    if (info.CurrentState == 1)
                    {
                        digitalIODataGridView.Rows[index].Cells[1].Style.BackColor = Color.LightGreen;
                    }
                    else
                    {
                        digitalIODataGridView.Rows[index].Cells[1].Style.BackColor = Color.White;
                    }
                }
            }

            // Enable the "Initialise Connection" button
            this.initialiseConnectionButton.Enabled = true;
        }

        /// <summary>
        /// Event handler: click on the start button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startButton_Click(object sender, EventArgs e)
        {
            int refreshTime = 0;

            // Get the refresh time
            try
            {
                refreshTime = Convert.ToInt32(this.refreshTimeTextBox.Text);
                if ((refreshTime < 10) || (refreshTime > 1000))
                    throw new Exception();
            }
            catch (Exception)
            {
                MessageBox.Show("Refresh time value is wrong. It must be a number between 10 and 1000.", 
                    "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // Update some graphics
            this.startButton.Enabled = false;
            this.stopButton.Enabled = true;
            this.actionToolStripProgressBar.Visible = true;
            this.actionToolStripStatusLabel.Text = "Monitor the values of the digital inputs.";
            this.actionToolStripStatusLabel.Visible = true;
            this.initialiseConnectionButton.Enabled = false;

            this.readBackgroundWorker.RunWorkerAsync(new ReadWorkerParam(refreshTime, msxe));
        }

        /// <summary>
        /// Event handler: click on the stop button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopButton_Click(object sender, EventArgs e)
        {
            this.stopButton.Enabled = false;
            readBackgroundWorker.CancelAsync();
        }

        /// <summary>
        /// Main job of the read background worker
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void readBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = (BackgroundWorker)sender;
            ReadWorkerParam p = (ReadWorkerParam)e.Argument;
            for (; ; )
            {
                int retval = 0;
                string error = string.Empty;

                if (worker.CancellationPending)
                {
                    e.Result = null;
                    return;
                }

                // Ask the update
                retval = msxe.UpdateCurrentDigitalIOState(out error);
                if (retval != 0)
                {
                    e.Result = error;
                    return;
                }

                // Report
                worker.ReportProgress(1);

                // Wait next
                System.Threading.Thread.Sleep(p.refreshTime);
            }
        }

        /// <summary>
        /// Event handler: read background worker report some progress
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void readBackgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            int index = 0;
            for (int i = 0; i < (int)this.msxe.DigIOChannelNb; i++)
            {
                DigitalIO info = this.msxe.GetDigitalIoInfo(i);
                // Not an input?
                if (info.CurrentConfiguration != 0)
                {
                    continue;
                }

                digitalIODataGridView.Rows[index].Cells[1].Value = info.CurrentState.ToString();
                if (info.CurrentState == 1)
                {
                    digitalIODataGridView.Rows[index].Cells[1].Style.BackColor = Color.LightGreen;
                }
                else
                {
                    digitalIODataGridView.Rows[index].Cells[1].Style.BackColor = Color.White;
                }
                index++;
            }
        }

        /// <summary>
        /// Event handler: read background worker is completed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void readBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Result != null)
            {
                // Display the error
                MessageBox.Show((string)e.Result, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            // Update graphics
            this.initialiseConnectionButton.Enabled = true;
            this.startButton.Enabled = true;
            this.actionToolStripProgressBar.Visible = false;
            this.actionToolStripStatusLabel.Text = "Connection opened with the system: " + msxe.IPAddress;
        }
    }
}