using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace SetDigitalOutputs
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// Store the MSX-E object
        /// </summary>
        private Msxexxxx msxe;

        /// <summary>
        /// Initialised?
        /// </summary>
        private bool initialised;

        /// <summary>
        /// Constructor of the graphical interface
        /// </summary>
        public MainForm()
        {
            this.initialised = false;

            InitializeComponent();
        }

        /// <summary>
        /// Event handler: occurs once when the form is loaded
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            // Load default parameters
            this.ipTextBox.Text = Properties.Settings.Default.ip;
        }

        /// <summary>
        /// Event handler: occurs when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Event handler: click on the "Initialise connection" button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void initialiseConnectionButton_Click(object sender, EventArgs e)
        {
            // Check the validity of the IP address
            if (this.ipTextBox.Valid == false)
                return;

            this.initialised = false;

            // Update some graphics
            this.initialiseConnectionButton.Enabled = false;
            this.actionToolStripProgressBar.Visible = true;
            this.actionToolStripStatusLabel.Text = "Initialise the connection with the MSX-E system.";
            this.actionToolStripStatusLabel.Visible = true;

            this.digitalIOGroupBox.Enabled = false;
            this.digitalIODataGridView.Rows.Clear();

            // Start the connection background worker
            this.connectionBackgroundWorker.RunWorkerAsync(this.ipTextBox.Text);
        }

        /// <summary>
        /// Job of the connection background worker
        /// The connection task is done in background since it may takes some time to initialise the system
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectionBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            int retval = 0;
            string error = string.Empty;

            // Construct the new MSX-E object
            Msxexxxx m = new Msxexxxx(this.ipTextBox.Text);

            // Do the connection
            retval = m.Connect(out error);
            e.Result = new ConnectionWorkerResult(m, retval, error);
        }

        /// <summary>
        /// Event handler: Connection background worker is completed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectionBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            ConnectionWorkerResult r = (ConnectionWorkerResult)e.Result;

            // Error ?
            if (r.retval != 0)
            {
                string error = string.Empty;
                this.actionToolStripProgressBar.Visible = false;
                this.actionToolStripStatusLabel.Text = "Error occurred.";
                // Reset the system type
                this.systemInformationTextBox.Text = string.Empty;
                // Display the right text
                switch (r.retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + r.error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + r.error;
                        break;
                }
                // Display the error
                MessageBox.Show(error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                // Store the object
                this.msxe = r.msxe;

                this.actionToolStripProgressBar.Visible = false;
                this.actionToolStripStatusLabel.Text = "Connection opened with the system: " + r.msxe.IPAddress;
                // Display the system type
                this.systemInformationTextBox.Text = r.msxe.SystemType;

                // Enable controls
                this.digitalIOGroupBox.Enabled = true;

                // Set data grid
                for (int i = 0; i < (int)this.msxe.DigIOChannelNb; i++)
                {
                    DigitalIO info = this.msxe.GetDigitalIoInfo(i);

                    // Only output
                    if (info.CurrentConfiguration != 1)
                        continue;

                    int index = digitalIODataGridView.Rows.Add();
                    digitalIODataGridView.Rows[index].Cells[0].Value = i.ToString();
                    digitalIODataGridView.Rows[index].Cells[2].Value = info.Port.ToString();

                    DataGridViewCheckBoxCell cell = (DataGridViewCheckBoxCell)digitalIODataGridView.Rows[index].Cells[1];
                    if ( info.CurrentState == 1)
                    {
                        cell.Value = true;
                        digitalIODataGridView.Rows[index].Cells[0].Style.BackColor = Color.LightGreen;
                    }
                }
            }

            // Enable the "Initialise Connection" button
            this.initialiseConnectionButton.Enabled = true;
            this.initialised = true;
        }

        /// <summary>
        /// Change the state of a value
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void digitalIODataGridView_CellValueChanged(object sender, DataGridViewCellEventArgs e)
        {
            if (this.initialised == false)
                return;

            DigitalIO output = this.msxe.GetDigitalOutputInfo(e.RowIndex);
            DataGridViewCheckBoxCell cell = (DataGridViewCheckBoxCell)digitalIODataGridView.Rows[e.RowIndex].Cells[1];
            ulong state = 0;
            int retval = 0;
            string error = string.Empty;

            // High or low?
            if ((Boolean)cell.Value == true)
            {
                state = 1;
                digitalIODataGridView.Rows[e.RowIndex].Cells[0].Style.BackColor = Color.LightGreen;
            }
            else
            {
                digitalIODataGridView.Rows[e.RowIndex].Cells[0].Style.BackColor = Color.White;
            }

            // Set the output
            retval = this.msxe.DigitalIOWriteChannel(output.Index, state, out error);
            if (retval != 0)
            {
                switch (retval)
                {
                    case -1:
                        error = "The selected IP address is wrong. Please change it and retry.";
                        break;
                    case -2:
                        error = "A system error occured: " + error;
                        break;
                    case -3:
                        error = "The selected system cannot be used with this sample.";
                        break;
                    case -4:
                        error = "It is not possible to open a connection with the selected system.";
                        break;
                    default:
                        error = "An error occured: " + error;
                        break;
                }

                // Display the error
                MessageBox.Show(error, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
        }

        /// <summary>
        /// Occurs when a check box is changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void digitalIODataGridView_CurrentCellDirtyStateChanged(object sender, EventArgs e)
        {
            if (this.initialised == false)
                return;

            if (digitalIODataGridView.IsCurrentCellDirty)
            {
                digitalIODataGridView.CommitEdit(DataGridViewDataErrorContexts.Commit);
            }
        }
    }
}